<?php

namespace LavenderBits\Squaby\Mail;

use Kirby\Cms\Page;
use LavenderBits\Squaby\DBService;
use LavenderBits\Squaby\Newsletter;
use LavenderBits\Squaby\Subscriber;

class Mail
{
    public $subject = "";
    public $template = "";
    public $data = "";
    public $recipient;
    private $newsletter;

    function __construct(
        Newsletter $newsletter,
        Subscriber $recipient,
        string $subject,
        string $template,
        array $data = []
    ) {
        $this->recipient = $recipient;
        $this->subject = $subject;
        $this->template = $template;
        $this->data = $data;
        $this->newsletter = $newsletter;
    }

    public function serialize(): array
    {
        return [
            "subject" => $this->subject,
            "template" => $this->template,
            "to" => $this->recipient->email()->toString(),
            "toName" => $this->recipient->name()->toString() ?? false,
            // TODO: Switch to transport driven fromName and from
            // We have custom transport settings (squaby.transport instead of email.transport)
            // So let's actually use them
            "from" => $this->newsletter->fromEmail()->toString(),
            "fromName" => $this->newsletter->fromName()->toString(),
            "data" => $this->data,
        ];
    }

    private static function createTrackingLink(Page $issue): string
    {
        $db = DBService::getDbConnection();
        $token = uuid();
        $issue = urlencode($issue->id());
        $db->table("tracking")->insert([
            "id" => $token,
            "issue_id" => $issue,
            "opened" => 0,
        ]);

        return join("/", [
            option("squaby.domain"), // TODO: Find a better way to get the website's domain while in CLI mode. This is gross.
            "newsletter-api",
            "pixel",
            $issue,
            $token,
        ]);
    }

    public static function buildForIssue(
        Page $issue,
        Subscriber $subscriber
    ): Mail {
        /** @var Newsletter */
        $newsletter = $issue->parent();

        // TODO: Fix title and add subtitle maybe
        return new Mail(
            $issue->parent(),
            $subscriber,
            $issue->title(),
            $issue->template()->name(),
            [
                "page" => $issue,
                "subscriber" => $subscriber,
                "unsubscribe" => $subscriber->createUnsubscribeLink(),
                "tracking" =>
                    "<img src=\"" . self::createTrackingLink($issue) . "\" />",
            ],
            $newsletter
        );
    }
}
