<?php

namespace LavenderBits\Squaby;

use Kirby\Cms\Page;
use Kirby\Toolkit\Date;
use Kirby\Toolkit\Str;
use LavenderBits\Squaby\Mail\Mail;
use LavenderBits\Squaby\Mail\Pigeon;

class Subscriber extends Page
{
    static function findBySecret(
        string $secret,
        Newsletter $newsletter
    ): Subscriber|false {
        return $newsletter->findSubscriber("secret", $secret);
    }

    public function confirm(): Subscriber
    {
        $newThis = $this->update([
            "hasOptedIn" => true,
            "activeSubscription" => true,
            "dateConfirmed" => Date::now(),
        ]);
        $newThis = $this->changeStatus("listed");
        return $newThis;
    }

    public function cancel(): Subscriber
    {
        $newThis = $this;
        $newThis = $newThis->update([
            "activeSubscription" => false,
            "dateUnsubscribed" => Date::now(),
        ]);
        $newThis = $newThis->changeStatus("unlisted");
        return $newThis;
    }

    public function sendConfirmationMail(): bool
    {
        $mail = new Mail(
            $this->parent(),
            $this,
            $this->parent()->optinMailTitle()->toString(),
            $this->parent()->optinMailTemplate()->toString(),
            ["confirmationLink" => $this->createConfirmationLink()]
        );
        return Pigeon::deliver($mail);
    }

    public function createConfirmationLink(): string
    {
        return join("/", [
            kirby()->site()->url(),
            "newsletter-api/confirm",
            $this->parent()->slug(),
            $this->secret()->toString(),
        ]);
    }

    public function createUnsubscribeLink(): string
    {
        return join("/", [
            kirby()->site()->url(),
            "newsletter-api/unsubscribe",
            $this->parent()->slug(),
            $this->secret()->toString(),
        ]);
    }
}
